/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import java.io.File;


/**
 * <p>
 * This class is a base class representing an application server to be used while running tests. The initializing data
 * includes the WAR (Web Archive) and EAR (Enterprise Archive) files.
 * </p>
 *
 * <p>
 * In the build file, this class represents the child element under the ServerElement. The name of the element is
 * supplied to the constructor. It is up to the subclass implementation on how to use it. The element supports the
 * following attributes, in addition to the attributes supported by the base class:
 *
 * <ul>
 * <li>
 * <b>warfile</b> - the Web Archive that contains the web application for running the tests. Can be absolute or
 * relative (to the project's basedir). Subclass should specify if this attribute is required.
 * </li>
 * <li>
 * <b>earfile</b> - the Enterprise Archive that contains the enterprise application for running the tests. Can be
 * absolute or relative (to the project's basedir). Subclass should specify if this attribute is required.
 * </li>
 * </ul>
 *
 * The "warfile" and "earfile" attributes should not be specified together. The nested elements supported by the base
 * class are inherited as well.
 * </p>
 *
 * <p>
 * For those methods called by the Ant framework, such as attribute setters, there is no check for arguments against
 * null. The Ant framework makes sure that the arguments will not be null.
 * </p>
 *
 * <p>
 * This class is not thread-safe. However the methods will be called one by one by the Ant framework according to the
 * life cycle of ant task. So thread-safety is not an issue here.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public abstract class ApplicationServer extends Server {
    /**
     * <p>
     * The "warfile" attribute specifies the Web Archive that contains the web application for running the tests.
     * Subclass should specify if this attribute is required. It is initially null, and set later by the setWarFile()
     * method.
     * </p>
     */
    private File warFile = null;

    /**
     * <p>
     * The "earfile" attribute specifies the Enterprise Archive that contains the enterprise application for running
     * the tests. Subclass should specify if this attribute is required. It is initially null, and set later by the
     * setEarFile() method.
     * </p>
     */
    private File earFile = null;

    /**
     * <p>
     * Creates a new instance of this class with the specified type. The type is the name of the element represented by
     * this class. This constructor simply calls the super constructor.
     * </p>
     *
     * @param type the type of the server.
     *
     * @throws IllegalArgumentException if type is null or empty string.
     * @throws BuildException if the type of the server is unrecognized.
     */
    protected ApplicationServer(String type) {
        super(type);
    }

    /**
     * <p>
     * Sets the value of the "warfile" attribute. The "warfile" attribute specifies the Web Archive that contains the
     * web application for running the tests.
     * </p>
     *
     * @param warFile the value of the "warfile" attribute.
     */
    public void setWarFile(File warFile) {
        this.warFile = warFile;
    }

    /**
     * <p>
     * Returns the value of the "warfile" attribute. The "warfile" attribute specifies the Web Archive that contains
     * the web application for running the tests.
     * </p>
     *
     * @return the value of the "warfile" attribute, or null if none is specified.
     */
    public File getWarFile() {
        return warFile;
    }

    /**
     * <p>
     * Sets the value of the "earfile" attribute. The "earfile" attribute specifies the Enterprise Archive that
     * contains the enterprise application for running the tests.
     * </p>
     *
     * @param earFile the value of the "earfile" attribute.
     */
    public void setEarFile(File earFile) {
        this.earFile = earFile;
    }

    /**
     * <p>
     * Returns the value of the "earfile" attribute. The "earfile" attribute specifies the Enterprise Archive that
     * contains the enterprise application for running the tests.
     * </p>
     *
     * @return the value of the "earfile" attribute, or null if none is specified.
     */
    public File getEarFile() {
        return earFile;
    }
}
