/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import org.apache.tools.ant.BuildException;

import java.util.Map;


/**
 * <p>
 * This class represents a piece of credentials information for accessing the server. It simply maps a property key to
 * a property value.
 * </p>
 *
 * <p>
 * In the build file, this class represents the child element under the "credentials" element. The name of the element
 * represents the property key, the value of the <b>value</b> attribute represents the property value.
 * </p>
 *
 * <p>
 * For those methods called by the Ant framework, such as attribute setters, there is no check for arguments against
 * null. The Ant framework makes sure that the arguments will not be null.
 * </p>
 *
 * <p>
 * This class is not thread-safe. However the methods will be called one by one by the Ant framework according to the
 * life cycle of ant task. So thread-safety is not an issue here.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class CredentialProperty {
    /**
     * <p>
     * Represents the name of the element represented by this class. It is set in the constructor and then never
     * changed. It cannot be null or empty string.
     * </p>
     */
    private final String name;

    /**
     * <p>
     * Represents the property map to add this credentials property into. It is a mapping from the property keys
     * (String) to property values (String). It is set in the constructor and the reference is never changed. A
     * property is added into the map by the setValue() method. It cannot be null.
     * </p>
     */
    private final Map propertyMap;

    /**
     * <p>
     * Creates a new instance of this class. The property map is not copied.
     * </p>
     *
     * @param name the name of the element.
     * @param propertyMap the property map to add this credentials property into.
     *
     * @throws IllegalArgumentException if name or propertyMap is null; or name is empty string.
     */
    public CredentialProperty(String name, Map propertyMap) {
        if (name == null) {
            throw new IllegalArgumentException("name is null");
        }
        if (name.trim().length() == 0) {
            throw new IllegalArgumentException("name is empty string");
        }
        if (propertyMap == null) {
            throw new IllegalArgumentException("propertyMap is null");
        }
        this.name = name;
        this.propertyMap = propertyMap;
    }

    /**
     * <p>
     * Sets the value of the "value" attribute. The "value" attribute represents the property value. This method adds
     * this credentials property to the property map.
     * </p>
     *
     * @param value the value of the "value" attribute, can be empty string.
     *
     * @throws BuildException if there are duplicates of the same credential in the property map.
     */
    public void setValue(String value) {
        // check if there are duplicates of the same credential
        if (propertyMap.containsKey(name)) {
            throw new BuildException("Only one nested " + name + " element is allowed");
        }
        propertyMap.put(name, value);
    }
}
