/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DynamicElementNS;
import org.apache.tools.ant.taskdefs.optional.junit.JUnitTask;
import org.apache.tools.ant.taskdefs.optional.junit.JUnitTest;

import java.io.File;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;


/**
 * <p>
 * This class is the main Ant task of the component. It is used to specify the common parameters for all the servers
 * and execute the tests.
 * </p>
 *
 * <p>
 * In the build file, this class represents the root task element. The name of the element is defined by the taskdef
 * core task. The element supports the following attribute, in addition to the standard attributes supported by
 * JUnitTask:
 *
 * <ul>
 * <li>
 * <b>reportsdir</b> (optional) - the directory to which the test reports should be written. Can be absolute or
 * relative (to the project's basedir). If not specified, the test reports will be written to the project's basedir.
 * </li>
 * </ul>
 *
 * Under this task element, there should be one or more server elements, each specifying the properties of the server.
 * Other nested elements supported by JUnitTask are inherited as well.
 * </p>
 *
 * <p>
 * Here is a snippet of the build file, showing the structure of the ServerAppTestTask:
 * <pre>
 * &lt;taskdef name="serverapptest" classname="com.topcoder.testframework.ServerAppTestTask"/&gt;
 *
 * &lt;!-- This Ant task starts the servers, runs the tests, and then stops the servers --&gt;
 * &lt;serverapptest printsummary="yes" failureproperty="tests.failed" todir="${test.reports.dir}"&gt;
 *
 *   &lt;classpath&gt;
 *     &lt;path refid="serverapptest.classpath"/&gt;
 *     &lt;pathelement location="${build.classes.dir}"/&gt;
 *     &lt;pathelement location="${test.classes.dir}"/&gt;
 *   &lt;/classpath&gt;
 *
 *   &lt;!-- This element describes the application server using Cactus back-end and Tomcat 5.5 server --&gt;
 *   &lt;com.topcoder.testframework.web.DefaultApplicationServer&gt;
 *     &lt;tomcat5x warfile="${test.dir}/test.war" dir="${tomcat5x.home}" port="8080"&gt;
 *       &lt;credentials&gt;
 *         &lt;user_name value="topcoder" /&gt;
 *         &lt;password value="123456" /&gt;
 *       &lt;/credentials&gt;
 *     &lt;/tomcat5x&gt;
 *   &lt;/com.topcoder.testframework.web.DefaultApplicationServer&gt;
 *
 *   &lt;!-- This element describes the database server (not provided by this component) --&gt;
 *   &lt;SomeDataBaseServer&gt;
 *     &lt;postgresql jarfile="${test.dir}/db_test_data.jar" dir="${postrgresql.home}" port="some_port_number"&gt;
 *     &lt;/postgresql&gt;
 *   &lt;/SomeDataBaseServer&gt;
 *
 *   &lt;formatter type="xml"/&gt;
 *   &lt;batchtest&gt;
 *     &lt;fileset dir="${src.test.dir}"&gt;
 *       &lt;include name="Test*.java"/&gt;
 *       &lt;exclude name="TestAll.java"/&gt;
 *     &lt;/fileset&gt;
 *   &lt;/batchtest&gt;
 *
 * &lt;/serverapptest&gt;
 * </pre>
 * </p>
 *
 * <p>
 * For those methods called by the Ant framework, such as attribute setters, there is no check for arguments against
 * null. The Ant framework makes sure that the arguments will not be null.
 * </p>
 *
 * <p>
 * This class is not thread-safe. However the methods will be called one by one by the Ant framework according to the
 * life cycle of ant task. So thread-safety is not an issue here.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class ServerAppTestTask extends JUnitTask implements DynamicElementNS {
    /**
     * <p>
     * The list of server elements (ServerElement) contained in the task represented by this class. It is initialized
     * in the constructor and the reference is never changed. The server elements are added to the list by the
     * createDynamicElement() method, and are retrieved in the execute() method. It cannot be null.
     * </p>
     */
    private final List servers;

    /**
     * <p>
     * The "reportsdir" attribute represents the directory to which the test reports should be written. This attribute
     * is optional. It is initially null, and set later by the setReportsDir() method.
     * </p>
     */
    private File reportsDir = null;

    /**
     * <p>
     * Creates a new instance of this class.
     * </p>
     *
     * @throws Exception if it fails to create the task, propagated from the super constructor.
     */
    public ServerAppTestTask() throws Exception {
        super();
        servers = new ArrayList();
    }

    /**
     * <p>
     * Executes the tests associated with this task. Basically it performs the following steps:
     *
     * <ol>
     * <li>
     * Start the servers.
     * </li>
     * <li>
     * Initialize the servers with data.
     * </li>
     * <li>
     * Run the test cases.
     * </li>
     * <li>
     * Clean up the data.
     * </li>
     * <li>
     * Stop the servers.
     * </li>
     * </ol>
     * </p>
     *
     * <p>
     * Note that the startUp(), initData(), cleanUpData() and shutDown() methods of the Server are invoked in a way
     * independent of the "startserver" and "initserver" attributes. These attributes should only affect the behavior
     * inside the methods. If any error happens, the process will simply stop at that point, and the resources held by
     * the servers behind are not released for debugging purpose.
     * </p>
     *
     * @throws BuildException if any required nested element is missing; or any error happens in the servers or test
     *         cases.
     */
    public void execute() {
        // check for required nested element
        if (servers.isEmpty()) {
            throw new BuildException("At least one server element must be specified");
        }

        for (Iterator i = servers.iterator(); i.hasNext();) {
            Server server = ((ServerElement) i.next()).getServer();

            // check for required server under the server element
            if (server == null) {
                throw new BuildException("A server must be specified under the server element "
                    + server.getClass().getName());
            }

            // start the server
            server.startUp();

            // initialize the server with data
            server.initData();
        }

        // set the reporting directory for the tests if specified
        if (reportsDir != null) {
            for (Enumeration en = getIndividualTests(); en.hasMoreElements();) {
                ((JUnitTest) en.nextElement()).setTodir(reportsDir);
            }
        }

        // run the test cases (depending on fork mode)
        super.execute();

        for (Iterator i = servers.iterator(); i.hasNext();) {
            Server server = ((ServerElement) i.next()).getServer();

            // clean up the data
            server.cleanUpData();

            // stop the server
            server.shutDown();
        }
    }

    /**
     * <p>
     * Creates a nested element with the specified name. The name specifies the fully qualified class name of the
     * Server implementation, thus it should be case-sensitive. The nested element must be of type ServerElement.
     * </p>
     *
     * <p>
     * Note that there can be one or more server elements of the same name. They are treated as different servers.
     * </p>
     *
     * @param uri the namespace uri of the nested element.
     * @param localName the local name of the nested element.
     * @param qName the qualified name of the nested element.
     *
     * @return the nested element, must be of type ServerElement.
     *
     * @throws IllegalArgumentException if localName is null or empty string.
     */
    public Object createDynamicElement(String uri, String localName, String qName) {
        ServerElement server = new ServerElement(localName);

        // pass the ant task information
        server.setAntTaskInfo(new AntTaskInfo(getProject(), getTaskName(), getLocation(), getOwningTarget()));

        // add the server element to the list
        servers.add(server);

        return server;
    }

    /**
     * <p>
     * Sets the value of the "reportsdir" attribute. The "reportsdir" attribute represents the directory to which the
     * test reports should be written.
     * </p>
     *
     * @param reportsDir the value of the "reportsdir" attribute.
     */
    public void setReportsDir(File reportsDir) {
        this.reportsDir = reportsDir;
    }

    /**
     * <p>
     * Returns the value of the "reportsdir" attribute. The "reportsdir" attribute represents the directory to which
     * the test reports should be written.
     * </p>
     *
     * @return the value of the "reportsdir" attribute, or null if none is specified.
     */
    public File getReportsDir() {
        return reportsDir;
    }
}
