/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import org.apache.cactus.integration.ant.CactifyWarTask;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.ZipFileSet;

import java.io.File;


/**
 * <p>
 * This class represents an Ant task that injects elements necessary to run the tests into an existing WAR (Web
 * Archive) file. It is used to prepare the WAR file for the DefaultWebApplicationServer. For now, it is a wrapper for
 * CactifyWarTask (in Cactus), but future implementations may change.
 * </p>
 *
 * <p>
 * In the build file, this class represents the root task element. The name of the element is defined by the taskdef
 * core task. The element supports the following attributes:
 *
 * <ul>
 * <li>
 * <b>srcfile</b> (required) - the source war file that is going to be cactified.
 * </li>
 * <li>
 * <b>destfile</b> (required) - the destination war file to create.
 * </li>
 * </ul>
 *
 * Under this task element, the <b>fileset</b>, <b>classes</b> and <b>lib</b> nested elements are used to specify jars,
 * classes and libs to be added to the war respectively.
 * </p>
 *
 * <p>
 * Here is a snippet of the build file, showing the structure of the PrepareTestWarTask:
 * <pre>
 * &lt;taskdef name="preparetestwar" classname="com.topcoder.testframework.PrepareTestWarTask"/&gt;
 *
 * &lt;!-- Prepare existing enterprise archive for testing --&gt;
 * &lt;preparetestwar srcfile="${target.dir}/${project.name.file}.war" destfile="${test.dir}/test.war"&gt;
 *   &lt;fileset dir="${src.test.dir}"&gt;
 *   &lt;/fileset&gt;
 * &lt;/preparetestwar&gt;
 * </pre>
 * </p>
 *
 * <p>
 * For those methods called by the Ant framework, such as attribute setters, there is no check for arguments against
 * null. The Ant framework makes sure that the arguments will not be null.
 * </p>
 *
 * <p>
 * This class is not thread-safe. However the methods will be called one by one by the Ant framework according to the
 * life cycle of ant task. So thread-safety is not an issue here.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class PrepareTestWarTask extends Task {
    /**
     * <p>
     * The wrapped CactifyWarTask instance to cactify the war file. It is initialized in the constructor and never
     * changed. Cannot be null.
     * </p>
     */
    private final CactifyWarTask cactifyWarTask;

    /**
     * <p>
     * Default constructor.
     * </p>
     */
    public PrepareTestWarTask() {
        super();
        cactifyWarTask = new CactifyWarTask();
    }

    /**
     * <p>
     * Sets the value of "srcfile" attribute. The "srcfile" attribute specifies the source war file that is going to be
     * cactified.
     * </p>
     *
     * @param srcFile the value of "srcfile" attribute.
     */
    public void setSrcFile(File srcFile) {
        cactifyWarTask.setSrcFile(srcFile);
    }

    /**
     * <p>
     * Adds a set of files, possibly jars to be added to the war.
     * </p>
     *
     * @param fileSet the files to add.
     *
     * @throws IllegalArgumentException if fileSet is null.
     */
    public void addFileset(FileSet fileSet) {
        if (fileSet == null) {
            throw new IllegalArgumentException("fileSet is null");
        }
        cactifyWarTask.addFileset(fileSet);
    }

    /**
     * <p>
     * Adds the class files under WEB-INF/classes/ to the war.
     * </p>
     *
     * @param fileSet the files to add.
     *
     * @throws IllegalArgumentException if fileSet is null.
     */
    public void addClasses(ZipFileSet fileSet) {
        if (fileSet == null) {
            throw new IllegalArgumentException("fileSet is null");
        }
        cactifyWarTask.addClasses(fileSet);
    }

    /**
     * <p>
     * Adds the library files under WEB-INF/lib/ to the war.
     * </p>
     *
     * @param fileSet the files to add.
     *
     * @throws IllegalArgumentException if fileSet is null.
     */
    public void addLib(ZipFileSet fileSet) {
        if (fileSet == null) {
            throw new IllegalArgumentException("fileSet is null");
        }
        cactifyWarTask.addLib(fileSet);
    }

    /**
     * <p>
     * Sets the value of "destfile" attribute. The "destfile" attribute specifies the destination war file to create.
     * </p>
     *
     * @param destFile the value of "destfile" attribute.
     */
    public void setDestFile(File destFile) {
        cactifyWarTask.setDestFile(destFile);
    }

    /**
     * <p>
     * Executes the task. Cactifies the source war file into destination war file. If "srcfile" or "destfile" attribute
     * is not specified, a BuildException will be thrown.
     * </p>
     *
     * @throws BuildException if it fails to prepare the war file.
     */
    public void execute() {
        cactifyWarTask.setProject(getProject());

        try {
            cactifyWarTask.execute();
        } catch (BuildException e) {
            // for BuildException, simply re-throw it
            throw e;
        } catch (Exception e) {
            // for other exceptions, wrap it with BuildException
            throw new BuildException("Fails to prepare war file", e);
        }
    }
}
