/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import org.apache.cactus.FilterTestCase;

import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * <p>
 * This class represents the test case to unit test Servlet Filters under the Testing Framework. Test classes that need
 * access to valid Filter API implicit objects must subclass this class. These implicit objects include: the HTTP
 * request, the HTTP response, the filter config, the filter chain. They should be accessed by the corresponding
 * getter methods.
 * </p>
 *
 * <p>
 * As in JUnit, you can define a setUp() and a tearDown() methods. They are executed respectively before and after each
 * test case. However they are executed on the server side. It means that you will be able to access the implicit
 * objects on the server only. The main method for a test is the testXXX() method which are executed in the server
 * container.
 * </p>
 *
 * <p>
 * For each XXX test case, you can define a corresponding beginXXX() method (optional). You will use it to initialize
 * HTTP related parameters (HTTP parameters, cookies, HTTP headers, URL to simulate, ...). Similarly, you can define a
 * corresponding endXXX() method (optional). You will use this method to verify the returned HTTP related parameters
 * from your test case (like the returned content of the HTTP response, any returned cookies, returned HTTP headers,
 * ...). The signatures of these methods are as follows:
 * <pre>
 * public void beginXXX(WebRequest theRequest) {
 *     [...]
 * }
 *  public void endXXX(WebResponse theResponse) {
 *     [...]
 * }
 * </pre>
 * where theRequest is the object that you use to set all the HTTP related parameters, theResponse is the object that
 * you use to verify the returned HTTP related parameters from your test case.
 * </p>
 *
 * <p>
 * No argument check is made in the constructor since it will be called by the JUnit framework. The passed argument
 * should always be valid.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class TCFilterTestCase extends FilterTestCase {
    /**
     * <p>
     * Default constructor.
     * </p>
     */
    public TCFilterTestCase() {
        super();
    }

    /**
     * <p>
     * Constructor with the name of the test case.
     * </p>
     *
     * @param name the name of the test case.
     */
    public TCFilterTestCase(String name) {
        super(name);
    }

    /**
     * <p>
     * Returns the config implicit object that will contain all initialization parameters defined in web.xml under the
     * Filter Redirector servlet definition.
     * </p>
     *
     * @return the config implicit object.
     */
    public FilterConfig getConfig() {
        return config;
    }

    /**
     * <p>
     * Returns the request implicit object that contains the HTTP related parameters set up in the beginXXX() method.
     * </p>
     *
     * @return the request implicit object.
     */
    public HttpServletRequest getRequest() {
        return request;
    }

    /**
     * <p>
     * Returns the response implicit object that contains the HTTP related parameters passed to the endXXX() method.
     * </p>
     *
     * @return the response implicit object.
     */
    public HttpServletResponse getResponse() {
        return response;
    }

    /**
     * <p>
     * Returns the implicit filterChain object that gives a view into the invocation chain of a filtered request for a
     * resource.
     * </p>
     *
     * @return the implicit filterChain object.
     */
    public FilterChain getFilterChain() {
        return filterChain;
    }
}
