/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.tools.ant.BuildFileTest;

import java.io.File;


/**
 * Unit tests for ServerAppTestTask implementation. The tests consist of two parts: one directly against the
 * ServerAppTestTask instance, the other indirectly through a sample build file for testing the execute() method.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class ServerAppTestTaskTest extends BuildFileTest {
    /**
     * The build file used to test for ServerAppTestTask.
     */
    private static final String BUILD_FILE = "test_files/ServerAppTestTask.xml";

    /**
     * The report file written by the ServerAppTestTask.
     */
    private static final String REPORT_FILE = "TEST-com.topcoder.testframework.SimpleTestCase.txt";

    /**
     * The ServerAppTestTask instance to test against.
     */
    private ServerAppTestTask task = null;

    /**
     * Creates a test of the specified name. Configures the sample build file to run the targets from.
     *
     * @param name the name of the test.
     */
    public ServerAppTestTaskTest(String name) {
        super(name);
        configureProject(BUILD_FILE);
    }

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(ServerAppTestTaskTest.class);

        return suite;
    }

    /**
     * Prepares a ServerAppTestTask instance for testing.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    protected void setUp() throws Exception {
        task = new ServerAppTestTask();
    }

    /**
     * Deletes the report files.
     */
    protected void tearDown() {
        new File(REPORT_FILE).delete();
        new File("test_files", REPORT_FILE).delete();
    }

    /**
     * Tests the accessibility of constructor.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor() throws Exception {
        new ServerAppTestTask();
    }

    /**
     * Test of execute method with one server specified. Verifies the execution sequence: start and initialize the
     * server, execute the test cases, clean up and stop the server.
     */
    public void testExecute_OneServer() {
        expectOutput("test_one_server",
            "type:startUp();type:initData();execute1;execute2;execute3;type:cleanUpData();type:shutDown();");
    }

    /**
     * Test of execute method with two servers specified. Verifies the execution sequence: start and initialize the
     * servers, execute the test cases, clean up and stop the servers.
     */
    public void testExecute_TwoServers() {
        expectOutput("test_two_servers",
            "type1:startUp();type1:initData();type2:startUp();type2:initData();execute1;execute2;execute3;"
            + "type1:cleanUpData();type1:shutDown();type2:cleanUpData();type2:shutDown();");
    }

    /**
     * Test of execute method with no reports dir specified. Verifies if the reports are written to the project base
     * dir.
     */
    public void testExecute_UnspecifiedReportsDir() {
        executeTarget("test_unspecified_reports_dir");
        assertTrue("Fails to write report to reports dir", new File(REPORT_FILE).exists());
    }

    /**
     * Test of execute method with reports dir specified. Verifies if the reports are written to the specified reports
     * dir.
     */
    public void testExecute_SpecifiedReportsDir() {
        executeTarget("test_specified_reports_dir");
        assertTrue("Fails to write report to reports dir", new File("test_files", REPORT_FILE).exists());
    }

    /**
     * Test of execute method with no server element under the task. Expects BuildException.
     */
    public void testExecute_NoServerElement() {
        expectBuildException("test_no_server_element", "At least one server element must be specified");
    }

    /**
     * Test of execute method with no server under the server element. Expects BuildException.
     */
    public void testExecute_NoServer() {
        expectBuildException("test_no_server",
            "A server must be specified under the server element " + SimpleServer.class.getName());
    }

    /**
     * Test of createDynamicElement method with null local name. Expects IllegalArgumentException.
     */
    public void testCreateDynamicElement_NullLocalName() {
        try {
            task.createDynamicElement("uri", null, "qName");
            fail("Creates nested element with null local name");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of createDynamicElement method with empty local name. Expects IllegalArgumentException.
     */
    public void testCreateDynamicElement_EmptyLocalName() {
        try {
            task.createDynamicElement("uri", "", "qName");
            fail("Creates nested element with empty local name");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getReportsDir and setReportsDir methods with reports dir specified. Verifies it by getting the reports
     * dir back after setting it.
     */
    public void testGetSetReportsDir_Specified() {
        File reportsDir = new File("");

        task.setReportsDir(reportsDir);
        assertEquals("Returns an incorrect reports dir", reportsDir, task.getReportsDir());
    }

    /**
     * Test of getReportsDir method with no reports dir specified. Should return null.
     */
    public void testGetReportsDir_Unspecified() {
        assertNull("Returns an incorrect reports dir", task.getReportsDir());
    }
}
