/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Location;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Target;


/**
 * Unit tests for ServerElement implementation.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class ServerElementTest extends TestCase {
    /**
     * The ServerElement instance to test against.
     */
    private ServerElement element = null;

    /**
     * The fully qualified class name of the Server implementation.
     */
    private String className = null;

    /**
     * The name of the nested element, which represents the server type.
     */
    private String name = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(ServerElementTest.class);

        return suite;
    }

    /**
     * Prepares a ServerElement instance for testing.
     */
    protected void setUp() {
        className = SimpleServer.class.getName();
        name = "type";
        element = new ServerElement(className);
    }

    /**
     * Tests the constructor with null class name. Expects IllegalArgumentException.
     */
    public void testConstructor_NullClassName() {
        try {
            new ServerElement(null);
            fail("Creates ServerElement with null class name");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Tests the constructor with empty class name. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyClassName() {
        try {
            new ServerElement("");
            fail("Creates ServerElement with empty class name");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of createDynamicElement method with null name. Expects IllegalArgumentException.
     */
    public void testCreateDynamicElement_NullName() {
        try {
            element.createDynamicElement(null);
            fail("Creates nested element with null name");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of createDynamicElement method with empty name. Expects IllegalArgumentException.
     */
    public void testCreateDynamicElement_EmptyName() {
        try {
            element.createDynamicElement("");
            fail("Creates nested element with empty name");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of createDynamicElement method with invalid class name. Expects BuildException.
     */
    public void testCreateDynamicElement_InvalidClassName() {
        element = new ServerElement("invalid");

        try {
            element.createDynamicElement(name);
            fail("Creates nested element with invalid class name");
        } catch (BuildException e) {
            // good
        }
    }

    /**
     * Test of createDynamicElement method with bad class name (not of type Server). Expects BuildException.
     */
    public void testCreateDynamicElement_BadClassName() {
        element = new ServerElement(String.class.getName());

        try {
            element.createDynamicElement(name);
            fail("Creates nested element with bad class name");
        } catch (BuildException e) {
            // good
        }
    }

    /**
     * Test of createDynamicElement method with non-existing server. Should return an element of type specified in the
     * constructor. Also verifies if the name is given to the constructor of the element.
     */
    public void testCreateDynamicElement_NonExistServer() {
        AntTaskInfo taskInfo = new AntTaskInfo(new Project(), "taskName", new Location("build.xml"), new Target());

        element.setAntTaskInfo(taskInfo);

        Object server = element.createDynamicElement(name);

        assertTrue("Returns an incorrect element", server instanceof SimpleServer);
        assertEquals("Fails to set the type of the server", name, ((SimpleServer) server).getType());
    }

    /**
     * Test of createDynamicElement method with existing server. Expects BuildException.
     */
    public void testCreateDynamicElement_ExistServer() {
        AntTaskInfo taskInfo = new AntTaskInfo(new Project(), "taskName", new Location("build.xml"), new Target());

        element.setAntTaskInfo(taskInfo);

        // make sure the server exists
        element.createDynamicElement(name);

        try {
            element.createDynamicElement(name);
            fail("Adds more than one server");
        } catch (BuildException e) {
            // good
        }
    }

    /**
     * Test of getServer method with no server specified. Should return null.
     */
    public void testGetServer_Unspecified() {
        assertNull("Returns an incorrect server", element.getServer());
    }

    /**
     * Test of getServer method with server specified. Should return the element from the createDynamicElement method.
     */
    public void testGetServer_Specified() {
        AntTaskInfo taskInfo = new AntTaskInfo(new Project(), "taskName", new Location("build.xml"), new Target());

        element.setAntTaskInfo(taskInfo);

        Server server = (Server) element.createDynamicElement(name);

        assertSame("Returns an incorrect server", server, element.getServer());
    }

    /**
     * Test of setAntTaskInfo method with null task info. Expects IllegalArgumentException.
     */
    public void testSetAntTaskInfo_NullTaskInfo() {
        try {
            element.setAntTaskInfo(null);
            fail("Sets null task info to server element");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getAntTaskInfo and setAntTaskInfo methods with non-null task info. Verifies it by getting the task info
     * back after setting it.
     */
    public void testGetSetAntTaskInfo_NonNullTaskInfo() {
        AntTaskInfo taskInfo = new AntTaskInfo(new Project(), "taskName", new Location("build.xml"), new Target());

        element.setAntTaskInfo(taskInfo);
        assertEquals("Returns an incorrect task info", taskInfo, element.getAntTaskInfo());
    }
}
